#!/usr/bin/env python
#----------------------------------------------------------------------------
# Project Name: MOSS
# File Name: moss.py
# Description: Run server.
#
# Created: 2004. 06. 20
# RCS-ID: $Id: moss.py,v 1.63 2004/08/04 05:36:31 myunggoni Exp $
# Copyright: (c) 2004 by myunggoni
# License: GNU General Public License
# Author: Myung-Gon, Park <myunggoni@users.kldp.net>
#----------------------------------------------------------------------------

__package__ = "MOSS"
__version__ = "0.1.2"

import ConfigParser
import sys
import getopt
import os
import live
import web

try:
	import ogg.vorbis
	ogg_support = "yes"
except ImportError:
	ogg_support = "no"

try:
	import signal
	signal_support = "yes"
except ImportError:
	signal_support = "no"

def sigterm_handler(signum, frame):
	global running
	
	running = None
	live.running = None
	
def sighup_handler(signum, frame):
	global reconfig
	
	reconfig = 1
	live.running = None
	
def daemonize(stdin = "/dev/null", stdout = "/dev/null", stderr = "/dev/null", pidfile = None):
	try: 
		pid = os.fork() 
		if pid > 0:
			sys.exit(0) # Exit first parent.
	except OSError, error: 
		sys.stdout.write("MOSS: Fork #1 failed(%s)\n" % (error.strerror))
		sys.exit(1)
        
	os.chdir("/") 
	os.umask(0) 
	os.setsid() 
   
	try: 
		pid = os.fork() 
		if pid > 0:
			sys.exit(0) # Exit second parent.
	except OSError, error: 
		sys.stdout.write("MOSS: Fork #2 failed(%s)\n" % (error.strerror))
		sys.exit(1)

	pid = str(os.getpid())
	
	if pidfile:
		try:
			file(pidfile, "w+").write("%s\n" % pid)
		except IOError, error:
			sys.stdout.write("MOSS: Invalid pidfile(%s)\n" % (error.strerror))
			sys.exit(1)
	
	sys.stdout.write("MOSS: Server started with pid %s\n" % pid)
	sys.stdout.flush()
	
	si = open(stdin, "r")
	so = open(stdout, "a+")
	se = open(stderr, "a+", 0)
	
	os.dup2(si.fileno(), sys.stdin.fileno())
	os.dup2(so.fileno(), sys.stdout.fileno())
	os.dup2(se.fileno(), sys.stderr.fileno())

def run_server(config_file, server_mode):
	global signal_support
	global reconfig
	global running 
	
	reconfig = None
	
	if signal_support == "yes":
		if "SIGTERM" in dir(signal):
			signal.signal(signal.SIGTERM, sigterm_handler)
		if "SIGHUP" in dir(signal):
			signal.signal(signal.SIGHUP, sighup_handler)
	
	if server_mode == "web":
		server = web.Server(config_file)
		server.log_message("MOSS: Server started with pid %d" % os.getpid())
		running = 1
		
		try:
			while running:
				if reconfig:
					server.log_message("MOSS: Reloading configuraion file")
					server.server_close()
					server = web.Server(config_file)
					reconfig = None

				server.handle_request()

			server.log_message("MOSS: Now taking down the server...")
			sys.exit(0)
		
		except KeyboardInterrupt:
			server.log_message("MOSS: Please wait while the remaining streams finish...")
			server.log_message("MOSS: KeyboardInterrupt occured")
			sys.exit(0)
	else:
		server = live.Server(config_file)
		server.log_message("MOSS: Server started with pid %d" % os.getpid())
		running = 1
		
		try:
			while running:
				if reconfig:
					server = live.Server(config_file)
					server.log_message("MOSS: Reloading configuration file")
					reconfig = None

				server.serve_forever()
		
			server.log_message("MOSS: Now taking down the server...")
			sys.exit(0)
		
		except KeyboardInterrupt:
			server.log_message("MOSS: KeyboardInterrupt occured")
			sys.exit(0)

def print_usage():
	print "Usage: %s [--help] [--debug] [--config <file name>] [--pidfile <file name>]" % os.path.basename(sys.argv[0])
	print "       -h      --help      Display this usage information."
	print "       -d      --debug     Run as a debug mode."
	print "       -c      --config    Specify the config file(default: %s)" % os.path.join(sys.prefix, "etc/moss.conf")
	print "       -p      --pidfile   Specify the pidfile, only if server runs as a daemon mode (default: %s)" % "/var/run/moss.pid"

def main():
	try:
		opts, args = getopt.getopt(sys.argv[1:], "hdc:p:", ["help", "debug", "config", "pidfile"])
	except getopt.GetoptError:
		print_usage()
		sys.exit(1)

	debug_mode = None
	config_file = os.path.join(sys.prefix, "etc/moss.conf")
	pidfile = "/var/run/moss.pid"

	for opt, arg in opts:
		if opt in ("-h", "--help"):
			print_usage()
			sys.exit(0)
		
		if opt in ("-d", "--debug"):
			debug_mode = 1
		
		if opt in ("-c", "--config"):
			config_file = arg
			
			if not os.path.isfile(config_file):
				print "MOSS: No such configuration file"
				print_usage()
				sys.exit(1)

		if opt in ("-p", "--pidfile"):
			pidfile = arg

	if os.path.isfile(config_file):
		config = ConfigParser.ConfigParser()
		config.add_section("server")
		config_defaults = config.defaults()
		config_defaults["server-mode"] = "web"
		config.read(config_file)
		server_mode = config.get("server", "server-mode")
	
		if server_mode in ("web", "live"):
			web.debug_mode = debug_mode
			live.debug_mode = debug_mode
		else:
			print "MOSS: Invalid server mode"
			sys.exit(1)
			
	else:
		print "MOSS: Configuration file not found"
		sys.exit(1)
	
	if ogg_support == "yes":
		print "MOSS: Ogg vorbis support enabled"
	else:
		print "MOSS: Ogg vorbis support disabled, to enable it you will need to install the 'pyogg' and the 'pyvorbis' modules"
			
	if not debug_mode:
		print "MOSS: Entering in daemon mode"
		config_file = os.path.realpath(config_file)
		pidfile = os.path.realpath(pidfile)
		daemonize("/dev/null", "/dev/null", "/dev/null", pidfile)
	else:
		print "MOSS: Entering in debug mode"
	
	run_server(config_file, server_mode)
			
if __name__ == "__main__":
	main()
